unit Sample11;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    EditSim: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    EditRec: TEdit;
    Label4: TLabel;
    EditFail: TEdit;
    Scheduler: TScheduler;
    DetailsSource: TGenerator;
    Bunker: TStack;
    Gate: TGate;
    Tool: TServer;
    AnotherWorkShop: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EditSimExit(Sender: TObject);
    procedure EditFailExit(Sender: TObject);
    procedure EditRecExit(Sender: TObject);
    procedure SchedulerBeforeTimeGoOn(Sender: TAggregate);
    procedure SchedulerPlanned(Sender: TAggregate);
    procedure DetailsSourceBeforeTimeGoOn(Sender: TAggregate);
    procedure DetailsSourceExit(Sender: TBlock; Trans: TTransaction);
    procedure DetailsSourceRouting(Sender: TBlock; Trans: TTransaction);
    procedure BunkerRouting(Sender: TBlock; Trans: TTransaction);
    procedure GateRouting(Sender: TBlock; Trans: TTransaction);
    procedure ToolEnter(Sender: TBlock; Trans: TTransaction);
    procedure ToolRelease(Sender: TBlock; Trans: TTransaction);
    procedure ToolRouting(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


var
  Form1: TForm1;
  LimitTime: real;           // Simulation time
  FailTime: real;            // Mean of time between falures
  RecoveryTime: real;        // Mean of recovery time


implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;

   try
      FailTime:=StrToFloat(Trim(EditFail.Text));
   except
      MessageDlg('Invalid time between failures',mtError,[mbOK],0);
      EditFail.SetFocus;
      Exit;
   end;
   if FailTime<=0.0 then
   begin
      MessageDlg('Invalid time between failures',mtError,[mbOK],0);
      EditFail.SetFocus;
   end;


   try
      RecoveryTime:=StrToFloat(Trim(EditRec.Text));
   except
      MessageDlg('Invalid recovery time',mtError,[mbOK],0);
      EditRec.SetFocus;
      Exit;
   end;
   if RecoveryTime<0.0 then
   begin
      MessageDlg('Invalid recovery time',mtError,[mbOK],0);
      EditRec.SetFocus;
   end;

   Memo.Lines.Clear;
   Gate.Unlock;
   Model.Simulate(LimitTime);
   Memo.Lines.Add('Throughput: '+FormatFloat('0.00',60*Tool.Exits/LimitTime)+
                  ' details/hour');
   Memo.Lines.Add('Average filling of the bunker: '+
                  FormatFloat('0.00',Bunker.AverageCount));
   Memo.Lines.Add('Maximum filling of the bunker: '+
                  IntToStr(Bunker.MaxCount));
   Model.Reset;
end;


procedure TForm1.EditSimExit(Sender: TObject);
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
   end;
end;

procedure TForm1.EditFailExit(Sender: TObject);
begin
   try
      FailTime:=StrToFloat(Trim(EditFail.Text));
   except
      MessageDlg('Invalid time between failures',mtError,[mbOK],0);
      EditFail.SetFocus;
      Exit;
   end;
   if FailTime<=0.0 then
   begin
      MessageDlg('Invalid time between failures',mtError,[mbOK],0);
      EditFail.SetFocus;
   end;
end;


procedure TForm1.EditRecExit(Sender: TObject);
begin
   try
      RecoveryTime:=StrToFloat(Trim(EditRec.Text));
   except
      MessageDlg('Invalid recovery time',mtError,[mbOK],0);
      EditRec.SetFocus;
      Exit;
   end;
   if RecoveryTime<0.0 then
   begin
      MessageDlg('Invalid recovery time',mtError,[mbOK],0);
      EditRec.SetFocus;
   end;
end;

procedure TForm1.SchedulerBeforeTimeGoOn(Sender: TAggregate);
begin
   Sender.NextTime(MultiRand.Exponential(FailTime));
end;

procedure TForm1.SchedulerPlanned(Sender: TAggregate);
begin
   if Gate.isLocked then
   begin
      Sender.NextTime(MultiRand.Exponential(FailTime));
      Gate.UnLock;
   end
   else
   begin
      Sender.NextTime(MultiRand.Exponential(RecoveryTime));
      Tool.Release;
      Gate.Lock;
   end;
end;


procedure TForm1.DetailsSourceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.DetailsSourceExit(Sender: TBlock; Trans: TTransaction);
begin
   Sender.NextTime(MultiRand.Exponential(2.0));
end;

procedure TForm1.DetailsSourceRouting(Sender: TBlock; Trans: TTransaction);
begin
   Sender.PassTo(Bunker);
end;

procedure TForm1.BunkerRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Gate);
end;

procedure TForm1.GateRouting(Sender: TBlock; Trans: TTransaction);
begin
   if Gate.isLocked then
      Sender.PassTo(Bunker)
   else
      Sender.PassTo(Tool);
end;

procedure TForm1.ToolEnter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(1.5));
end;

procedure TForm1.ToolRelease(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(Bunker);
end;

procedure TForm1.ToolRouting(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(AnotherWorkshop);
end;

procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;




end.
